﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ElectricLocomotive
{
    public class DrawingElectricLocomotive
    {
        public EntityElectricLocomotive? EntityElectricLocomotive;

        private int _pictureWidth;
        private int _pictureHeight;
        private int _startPosX;
        private int _startPosY;
        private readonly int _locoWidth = 150;
        private readonly int _locoHeight = 50;
        
        public bool Init(int speed, double weight, Color bodyColor, Color additionalColor, bool horns, bool seifbatteries,
            int width, int height)
        {

            if (width < _locoWidth || height < _locoHeight)
            {
                return false;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityElectricLocomotive = new EntityElectricLocomotive();
            EntityElectricLocomotive.Init(speed, weight, bodyColor, additionalColor, horns, seifbatteries);
            return true;
        }

        public void SetPosition(int x, int y)
        {
            if (x < 0 || x + _locoWidth > _pictureWidth)
            {
                x = 20;
            }
            if (y < 0 || y + _locoHeight > _pictureHeight)
            {
                y = 20;
            }
            _startPosX = x;
            _startPosY = y;
        }

        public void MoveTransport(DirectionType direction)
        {
            if (EntityElectricLocomotive == null)
            {
                return;
            }
            switch (direction)
            {
                case DirectionType.Left:
                    if (_startPosX - EntityElectricLocomotive.Step > 0)
                    {
                        _startPosX -= (int)EntityElectricLocomotive.Step;
                    }
                    break;
                case DirectionType.Up:
                    if (_startPosY - EntityElectricLocomotive.Step > 0)
                    {
                        _startPosY -= (int)EntityElectricLocomotive.Step;
                    }
                    break;
                case DirectionType.Right:
                    if (_startPosX + EntityElectricLocomotive.Step + _locoWidth < _pictureWidth)
                    {
                        _startPosX += (int)EntityElectricLocomotive.Step;
                    }
                    break;
                case DirectionType.Down:
                    if (_startPosY + EntityElectricLocomotive.Step + _locoHeight < _pictureHeight)
                    {
                        _startPosY += (int)EntityElectricLocomotive.Step;
                    }
                    break;
            }
        }
        public void DrawTransport(Graphics g)
        {
            if (EntityElectricLocomotive == null)
            {
                return;
            }

            Pen pen = new(Color.Black);
            Brush blackBrush = new SolidBrush(Color.Black);
            Brush windows = new SolidBrush(Color.LightBlue);
            Brush bodyColor = new SolidBrush(EntityElectricLocomotive.BodyColor);
            Brush additionalBrush = new SolidBrush(EntityElectricLocomotive.AdditionalColor);

            if (EntityElectricLocomotive.Horns)
            {
                //horns
                g.FillRectangle(blackBrush, _startPosX + 30, _startPosY + 15, 20, 5);
                g.DrawLine(pen, _startPosX + 40, _startPosY + 15, _startPosX + 50, _startPosY + 10);
                g.DrawLine(pen, _startPosX + 50, _startPosY + 10, _startPosX + 45, _startPosY);
                g.DrawLine(pen, _startPosX + 45, _startPosY + 15, _startPosX + 50, _startPosY + 10);
                g.DrawLine(pen, _startPosX + 50, _startPosY + 10, _startPosX + 40, _startPosY);
            }

            //локомотив
            g.FillPolygon(bodyColor, new Point[]
                {
                    new Point(_startPosX, _startPosY + 40),
                    new Point(_startPosX, _startPosY + 30),
                    new Point(_startPosX + 20, _startPosY + 20),
                    new Point(_startPosX + 70, _startPosY + 20),
                    new Point(_startPosX +80, _startPosY + 30),
                    new Point(_startPosX +80, _startPosY + 40),
                    new Point(_startPosX +75, _startPosY + 45),
                    new Point(_startPosX +5, _startPosY + 45),
                    new Point(_startPosX, _startPosY + 40),
                }
            );

            if (EntityElectricLocomotive.Seifbatteries)
            {
                g.DrawPolygon(pen, new Point[]
                {
                    new Point(_startPosX + 40, _startPosY + 25),
                    new Point(_startPosX + 65, _startPosY + 25),
                    new Point(_startPosX + 65, _startPosY + 35),
                    new Point(_startPosX + 40, _startPosY + 35),
                    new Point(_startPosX + 40, _startPosY + 25),
                }
            );
            }

            g.DrawPolygon(pen, new Point[]
                {
                    new Point(_startPosX, _startPosY + 40),
                    new Point(_startPosX, _startPosY + 30),
                    new Point(_startPosX + 20, _startPosY + 20),
                    new Point(_startPosX + 70, _startPosY + 20),
                    new Point(_startPosX +80, _startPosY + 30),
                    new Point(_startPosX +80, _startPosY + 40),
                    new Point(_startPosX +75, _startPosY + 45),
                    new Point(_startPosX +5, _startPosY + 45),
                    new Point(_startPosX, _startPosY + 40),
                }
            );

            //окошки
            g.FillPolygon(windows, new Point[]
                {
                    new Point(_startPosX + 10, _startPosY + 30),
                    new Point(_startPosX +15, _startPosY + 25),
                    new Point(_startPosX + 20, _startPosY + 25),
                    new Point(_startPosX + 20, _startPosY + 30),
                    new Point(_startPosX +10, _startPosY + 30),
                }
            );

            g.DrawPolygon(pen, new Point[]
                {
                    new Point(_startPosX + 10, _startPosY + 30),
                    new Point(_startPosX +15, _startPosY + 25),
                    new Point(_startPosX + 20, _startPosY + 25),
                    new Point(_startPosX + 20, _startPosY + 30),
                    new Point(_startPosX +10, _startPosY + 30),
                }
            );

            g.FillRectangle(windows, _startPosX + 25, _startPosY + 25, 10, 5);
            g.DrawRectangle(pen, _startPosX + 25, _startPosY + 25, 10, 5);

            //обязательные колеса
            //loco
            g.FillEllipse(blackBrush, _startPosX + 10, _startPosY + 45, 5, 5);
            g.FillEllipse(blackBrush, _startPosX + 25, _startPosY + 45, 5, 5);
            g.FillEllipse(blackBrush, _startPosX + 50, _startPosY + 45, 5, 5);
            g.FillEllipse(blackBrush, _startPosX + 65, _startPosY + 45, 5, 5);

            //telega
            g.FillEllipse(blackBrush, _startPosX + 95, _startPosY + 45, 5, 5);
            g.FillEllipse(blackBrush, _startPosX + 140, _startPosY + 45, 5, 5);

            g.FillEllipse(blackBrush, _startPosX + 130, _startPosY + 45, 5, 5);
            g.FillEllipse(blackBrush, _startPosX + 105, _startPosY + 45, 5, 5);

            //telejka
            g.FillPolygon(additionalBrush, new Point[]
                {
                    new Point(_startPosX + 90, _startPosY + 25),
                    new Point(_startPosX + 95, _startPosY + 20),
                    new Point(_startPosX + 145, _startPosY + 20),
                    new Point(_startPosX + 150, _startPosY + 25),
                    new Point(_startPosX + 150, _startPosY + 45),
                    new Point(_startPosX + 90, _startPosY + 45),
                    new Point(_startPosX + 90, _startPosY + 25),
                }
            );

            g.DrawPolygon(pen, new Point[]
                {
                    new Point(_startPosX + 90, _startPosY + 25),
                    new Point(_startPosX + 95, _startPosY + 20),
                    new Point(_startPosX + 145, _startPosY + 20),
                    new Point(_startPosX + 150, _startPosY + 25),
                    new Point(_startPosX + 150, _startPosY + 45),
                    new Point(_startPosX + 90, _startPosY + 45),
                    new Point(_startPosX + 90, _startPosY + 25),
                }
            );
            //телега окна
            g.DrawLine(pen, _startPosX + 80, _startPosY + 40, _startPosX + 90, _startPosY + 40);
            g.FillRectangle(windows, _startPosX + 95, _startPosY + 30, 10, 5);
            g.FillRectangle(windows, _startPosX + 115, _startPosY + 30, 10, 5);
            g.FillRectangle(windows, _startPosX + 135, _startPosY + 30, 10, 5);
        }
    }
}